/*****************************************************************************
 * devicemanager.c: Device Manager scanner core code
 *****************************************************************************
 * $Id: devicemanager.c,v 1.25 2008/03/22 17:05:39 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2007 Alexis Ballier
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include "devicemanager.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dirent.h>

#ifndef DEVMAN_STANDALONE
#include "grab.h"
#endif

struct devman_device_list* devman_devices = NULL;
struct devman_device_class_list* all_devices_classes = NULL;

#define	SYSCLASS_BASEDIR	"/sys/class/"

#ifndef DEVMAN_STANDALONE
extern int debug;
#else
int debug = 1;
#endif

int devman_wanted = 0;


/* default grabber that devman will activate */
char devman_grabber_dvb[]="dvb";
char devman_grabber_v4l2[]="v4l2";
char devman_grabber_v4l1[]="video4linux";
char devman_grabber_dummy[]="dummy";


#ifdef HAVE_DVB
 #define DEVMAN_DEFAULT_GRABBER	devman_grabber_dvb
#else
 #ifdef HAVE_V4L_2
  #define DEVMAN_DEFAULT_GRABBER	devman_grabber_v4l2
 #else
  #ifdef HAVE_V4L_1
   #define DEVMAN_DEFAULT_GRABBER	devman_grabber_v4l1
  #else
   #define DEVMAN_DEFAULT_GRABBER	devman_grabber_dummy
  #endif
 #endif
#endif

char* devman_default_grabber = DEVMAN_DEFAULT_GRABBER;


void devman_device_list_clean(struct devman_device_list* toclean){
	if(toclean == NULL) return;
	free(toclean->current);
	devman_device_list_clean(toclean->next);
	free(toclean);
}

int devman_add_device_to_device_list(struct devman_device* device_to_add){
	struct devman_device_list* temp_devman_devices;

	temp_devman_devices = (struct devman_device_list*)malloc(sizeof(struct devman_device_list));
	temp_devman_devices->current = device_to_add;
	temp_devman_devices->next = devman_devices;
	devman_devices = temp_devman_devices;


	if(debug){
		fprintf(stderr, "Adding device : %s\n major : %d, minor: %d\n precise type: %s\n device location: %s\n device id: %d\n", device_to_add->name, device_to_add->major, device_to_add->minor, device_to_add->precise_type, device_to_add->device_location, device_to_add->device_id);
	}

	return 1;
}


/*
 * * convenience helper function for scandir's |compar()| function: 
 * * sort directory entries using strcoll(3)
 * */
int
devman_alphasort(const void *_a, const void *_b)
{
	struct dirent **a = (struct dirent **)_a;
	struct dirent **b = (struct dirent **)_b;
	return strcoll((*a)->d_name, (*b)->d_name);
}


int devman_scanned_directory( const struct dirent * entry ){

	if ((strcmp(entry->d_name, ".") == 0) || (strcmp(entry->d_name, "..") == 0))
		return 0;
	else return 1;
}


int devman_add_one_device(const char* sysclass_dir, int type, const char* dir){
	int i,j, count_files, fs_res, fr_res;
	struct dirent **files;
	char final_file[DEVMAN_MAX_CLASSDIR_LENGTH];
	FILE* opened_file;
	struct devman_device* mydevice;

	mydevice = (struct devman_device*)malloc(sizeof(struct devman_device));



	switch(type){
		case DEVMAN_V4L:
			/* scanning dir name to get the device name */
			for(i=0; (dir[i] < '0' || dir[i] > '9') && dir[i] && i < DEVMAN_MAX_NAME_LENGTH; i++){
				mydevice->precise_type[i]=dir[i];
			}
			mydevice->precise_type[i]='\0';

			sscanf(dir+i, "%i", &(mydevice->device_id));
			snprintf(mydevice->device_location, DEVMAN_MAX_NAME_LENGTH, "/dev/v4l/%s",dir);
			break;
#ifdef HAVE_DVB
		case DEVMAN_DVB:
			/* scanning dir name to get the device name */
			for(i=0; (dir[i] < '0' || dir[i] > '9') && dir[i] && i < DEVMAN_MAX_NAME_LENGTH; i++){
				mydevice->precise_type[i]=dir[i];
			}
			mydevice->precise_type[i]='\0';

			sscanf(dir+i, "%i", &(mydevice->device_id));
			i+=2;
			for(j=0; (dir[i+j] < '0' || dir[i+j] > '9') && dir[i+j] && i+j < DEVMAN_MAX_NAME_LENGTH; j++){
				mydevice->precise_type[j]=dir[i+j];
			}
			mydevice->precise_type[j]='\0';

			snprintf(mydevice->device_location, DEVMAN_MAX_NAME_LENGTH, "/dev/dvb/adapter%i/%s%i",mydevice->device_id,mydevice->precise_type,mydevice->device_id);

			snprintf(mydevice->name, DEVMAN_MAX_NAME_LENGTH, "DVB:%s",dir);
			break;
#endif
		default:
			break;
	}


	count_files = scandir(sysclass_dir, &files, devman_scanned_directory, devman_alphasort);
	for(i=0; i < count_files;i++){
		snprintf(final_file, DEVMAN_MAX_CLASSDIR_LENGTH, "%s/%s",sysclass_dir,files[i]->d_name);
		if ((strcmp(files[i]->d_name, "dev") == 0)){
			opened_file = fopen(final_file, "r");
			if(opened_file){
				fs_res = fscanf(opened_file, "%d:%d", &(mydevice->major), &(mydevice->minor));
				fclose(opened_file);
			}
		}
		if ((strcmp(files[i]->d_name, "name") == 0)){
			opened_file = fopen(final_file, "r");
			if(opened_file){
				fr_res = fread(mydevice->name, sizeof(char), DEVMAN_MAX_NAME_LENGTH  , opened_file);
				/*Just some cleaning... */
				for(j=0; j < DEVMAN_MAX_NAME_LENGTH ;j++){
					if(mydevice->name[j] == '\n'){
						mydevice->name[j]=0;
						j = DEVMAN_MAX_NAME_LENGTH+1;
					}
				}
			fclose(opened_file);
			}
		}
	}

	mydevice->device_type = type;

	return devman_add_device_to_device_list(mydevice);

}

int devman_add_devices(struct devman_device_class* scanned_devices){
	struct dirent **dirs;
	int count_dir,i;
	char final_dir[DEVMAN_MAX_CLASSDIR_LENGTH];
	int number_added_devices = 0;

	if(debug){
		fprintf(stderr, "Adding devices from : %s\n", scanned_devices->device_class_dir);
	}


	count_dir = scandir(scanned_devices->device_class_dir, &dirs, devman_scanned_directory, devman_alphasort);

	/*Now we add the devices */
	for(i=0; i < count_dir; i++){
		snprintf(final_dir, DEVMAN_MAX_CLASSDIR_LENGTH, "%s/%s",scanned_devices->device_class_dir,dirs[i]->d_name);
		number_added_devices += devman_add_one_device(final_dir, scanned_devices->type, dirs[i]->d_name);
	}
	return number_added_devices;
}



int devman_scan_devices(){
	int number_of_devices_found = 0;
	struct devman_device_class_list* temp_classes = all_devices_classes;

	if(devman_devices != NULL){
	 /* Rescanning, cleaning the list of devices */
		devman_device_list_clean(devman_devices);
	}

	while(temp_classes != NULL){
		number_of_devices_found += devman_add_devices(temp_classes->current);
		temp_classes = temp_classes->next;
	}
	

	if(number_of_devices_found > 0) return DEVMAN_SCAN_SUCCESS;
	else return DEVMAN_NO_DEVICE_FOUND;

}


int devman_init(){
	struct devman_device_class* device_to_add;
	int i;

	if(debug) fprintf(stderr, "Initializing devman\n");
	for(i=0; i < DEVMAN_DEVICE_TYPE_NUMBER ;i++){
		device_to_add = (struct devman_device_class*)malloc(sizeof(struct devman_device_class));
		device_to_add->type = i;
		switch(i){
		case DEVMAN_V4L:
			snprintf(device_to_add->device_class_dir,DEVMAN_MAX_CLASSDIR_LENGTH ,"%s%s", SYSCLASS_BASEDIR, "video4linux");
			devman_register_new_device_class(device_to_add);
			break;
#ifdef HAVE_DVB
		case DEVMAN_DVB:
			snprintf(device_to_add->device_class_dir,DEVMAN_MAX_CLASSDIR_LENGTH ,"%s%s", SYSCLASS_BASEDIR, "dvb");
			devman_register_new_device_class(device_to_add);
			break;
#endif
		default:
			fprintf(stderr, "Error in devman_init : device unknown\n");
			return DEVMAN_FAILED;
		}

	}



	return devman_read_config_file();

}


int devman_register_new_device_class(struct devman_device_class* newdeviceclass){
	struct devman_device_class_list* temp_devices_classes;

	temp_devices_classes = (struct devman_device_class_list*)malloc(sizeof(struct devman_device_class_list));
	temp_devices_classes->next = all_devices_classes;
	temp_devices_classes->current = newdeviceclass;
	all_devices_classes = temp_devices_classes;
	return DEVMAN_SUCCESS;
}


struct devman_device_list* devman_device_matching_type(const char* type, struct devman_device_list* current_devman_devices){
	struct devman_device_list* temp_devman_devices = current_devman_devices;

	while(temp_devman_devices != NULL){
		if(strcmp(type, temp_devman_devices->current->precise_type) == 0) return temp_devman_devices;
		temp_devman_devices = temp_devman_devices->next;
	}
	return temp_devman_devices;
}

struct devman_device* devman_device_mathing_type_from_number(const char* type, int number){
	int i = 0;
	struct devman_device_list* temp_devman_devices;
	temp_devman_devices = devman_device_matching_type(type, devman_devices);
	for(i = 0;(temp_devman_devices != NULL) && (i < number); temp_devman_devices = devman_device_matching_type(type, temp_devman_devices->next)) i++;

	if(temp_devman_devices == NULL) return NULL;
	else return temp_devman_devices->current;

}

struct devman_device* devman_device_matching_name(const char* name){
	struct devman_device_list* temp_devman_devices = devman_devices;

	while(temp_devman_devices != NULL){
		if(strcmp(name, temp_devman_devices->current->name) == 0) return temp_devman_devices->current;
		temp_devman_devices = temp_devman_devices->next;
	}

	return NULL;
}
	
void devman_initialize_default_grabber(){

	struct devman_device* mydevice = NULL;
	if(devman_config_params->device_name != NULL) mydevice = devman_device_matching_name(devman_config_params->device_name);

	if(mydevice != NULL){
		switch(mydevice->device_type){
			case DEVMAN_V4L:
#ifndef DEVMAN_STANDALONE
				device.video = mydevice->device_location;
#endif
#ifdef HAVE_V4L_2
				devman_default_grabber = devman_grabber_v4l2;
				return;
#else
#ifdef HAVE_V4L_1
				devman_default_grabber = devman_grabber_v4l1;
				return;
#endif
#endif
				break;
#ifdef HAVE_DVB
			case DEVMAN_DVB:
				devman_default_grabber = devman_grabber_dvb;
				return;
#endif
				break;
			default:
				break;
		}
	}
	
	fprintf(stderr, "No suitable device found, will use dummy grabber\n");
	devman_default_grabber = devman_grabber_dummy;
}



int devman_full_init(){
	int i;
	i = devman_init();
	if(i != DEVMAN_SUCCESS) return i;
	if(devman_scan_devices() == DEVMAN_NO_DEVICE_FOUND) return DEVMAN_FAILED;

	devman_initialize_default_grabber();

	return DEVMAN_SUCCESS;

}

#ifdef DEVMAN_STANDALONE
struct devman_device* devman_cmdline_selector(const char* type){
	int number_of_devices, input;
	struct devman_device_list* temp_devman_devices;

	printf("Choose between the following devices:\n");
	temp_devman_devices = devman_device_matching_type(type, devman_devices);
	for(number_of_devices = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type(type, temp_devman_devices->next)){
			printf("%d : %s (%s) \n", number_of_devices, temp_devman_devices->current->name, temp_devman_devices->current->device_location);
			number_of_devices++;
	}
	printf("%d : None\n", number_of_devices);
	
	scanf("%d", &input);

	return devman_device_mathing_type_from_number(type, input);
}

int main(){
	struct devman_device* test;
	char c;

	printf("devman_init() returned : %d\n", devman_init() );


	if(devman_config_params->device_name != NULL){
		printf("Hey, you have a config file !\nWe found %s as your selected device\nWant to change it? (Y/N)\n", devman_config_params->device_name);		
		scanf("%c", &c);
		if(c == 'N') return 0;
	}

	printf("devman_scan_devices() returned : %d\n", devman_scan_devices() );
	
	test = devman_cmdline_selector("video");

	if(test != NULL){
		printf("Choosen device :  %s (%s) \n", test->name, test->device_location);
		devman_config_params->device_name = test->name;
	}
	else{
		printf("No device selected!\n");
		devman_config_params->device_name = NULL;
	}
	
	devman_initialize_default_grabber();
/*
	test = devman_cmdline_selector("vbi");

	if(test != NULL)
		printf("Choosen device :  %s (%s) \n", test->name, test->device_location);
	else printf("No device selected!\n");
*/
	printf("Choosen grabber : %s\n",  devman_default_grabber);

	devman_write_config_file();
	return 0;
}
#endif
