/*****************************************************************************
 * devicemanager.h: Device Manager scanner header
 *****************************************************************************
 * $Id: devicemanager.h,v 1.18 2007/05/13 16:14:15 aballier Exp $
 *****************************************************************************
 * Copyright (C) 2007 Alexis Ballier
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/


#ifndef __DEVICEMANAGER_H
#define __DEVICEMANAGER_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

  

extern char* devman_default_grabber;



/* Maximum length of device names */
#define DEVMAN_MAX_NAME_LENGTH	100

/* Maximum length of a device class directory  */

#define	DEVMAN_MAX_CLASSDIR_LENGTH	200



/* Different device types */
enum{
	DEVMAN_V4L,
#ifdef HAVE_DVB
	DEVMAN_DVB,
#endif
	DEVMAN_DEVICE_TYPE_NUMBER
};

struct devman_device{

	char name[DEVMAN_MAX_NAME_LENGTH];

	int device_type;

	int major, minor;

	char precise_type[DEVMAN_MAX_NAME_LENGTH];
	char device_location[DEVMAN_MAX_NAME_LENGTH];
	int device_id; /* Only useful for dvb */
};



struct devman_device_list{
	struct devman_device* current;
	struct devman_device_list* next;
};


struct	devman_device_class{
	char device_class_dir[DEVMAN_MAX_CLASSDIR_LENGTH];
	int	type;
};

struct devman_device_class_list{
	struct devman_device_class* current;
	struct devman_device_class_list* next;
};


extern struct devman_device_list* devman_devices;
extern struct devman_device_class_list* all_devices_classes;

/* Return codes for devman_scan_devices */

#define DEVMAN_NO_DEVICE_FOUND	0
#define	DEVMAN_SCAN_SUCCESS	1


int devman_scan_devices();



/* devman procedures return values */

#define	DEVMAN_SUCCESS	0
#define DEVMAN_FAILED	1

int devman_init();

int devman_register_new_device_class(struct devman_device_class*);

/* Function to get the next device matching type */
struct devman_device_list* devman_device_matching_type(const char* type, struct devman_device_list* current_devman_devices);


struct devman_device* devman_device_mathing_type_from_number(const char* type, int number);
struct devman_device* devman_device_matching_name(const char* name);

#ifdef DEVMAN_STANDALONE
/* Command line device selector */
struct devman_device* devman_cmdline_selector(const char*);
#endif


/* Global variable to know if devman is wanted or not */
extern int devman_wanted;


/* Functions to read/write the config file */

int devman_read_config_file();
int devman_write_config_file();

struct devman_conf_params{
	char* device_name;
};

extern struct devman_conf_params* devman_config_params;


/* Grabber strings */

/* not needed for now...
extern char devman_grabber_dvb[];
extern char devman_grabber_v4l2[];
extern char devman_grabber_v4l1[];
extern char devman_grabber_dummy[];
*/


void devman_initialize_default_grabber();



/* This function is for lazy people like me that want a full
 * initialization only */

int devman_full_init();

#endif
