/*****************************************************************************
 * devicemanager_ui.c: Device Manager Parameters Xaw GUI
 *****************************************************************************
 * $Id: devicemanager_ui.c,v 1.11 2007/07/20 17:22:32 pingus77 Exp $
 *****************************************************************************
 * Copyright (C) 2007 Pingus
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>

#include "config.h"

#include "devicemanager.h"
#include "grab.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/neXtaw/XawInit.h>
#include <X11/neXtaw/Command.h>
#include <X11/neXtaw/Label.h>
#include <X11/neXtaw/Paned.h>
#include <X11/neXtaw/AsciiText.h>

#ifdef HAVE_XPM
# include <X11/xpm.h>
#endif

#include "../strtab.h"
#include "../xdtv.h"
#include "../grab.h"
#include "../x11.h"
#include "../toolbox.h"
# include "../popup_ui.h"

#ifdef HAVE_XPM
# include "../xdtv.xpm"
#endif

// POUR PLUS TARD QUAND CE SERA STABILISE
// internationalization
#include "../translation.h"
//#include "pixmaps_loader.h"

/*-------------------------------------------------------------------------*/
extern void save_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern void OptionsAction (Widget, XEvent *, String *, Cardinal *);
extern void action_cb (Widget widget, XtPointer clientdata, XtPointer call_data);
extern Widget app_shell;
extern Display *dpy;
extern Atom wm_protocols[2];
extern int debug;

/* Widgets which are not static are used into the XdTV pixmaps lib */
static Widget devicemanager_shell;
static Widget devicemanager_paned;
static Widget c_devicemanager_label;
static Widget c_devicemanager_api;
static Widget c_devicemanager_v4l;
static Widget c_devicemanager_vvideo;
static Widget c_devicemanager_vvbi;
static Widget c_devicemanager_dvbt;
static Widget c_devicemanager_tadapter;
static Widget c_devicemanager_switch;
static Widget c_devicemanager_openmain;
static Widget c_devicemanager_save;

// Pour mettre qqchose derrière les listes: sera surement modifié par ton code
// v = v4l ; t = tnt ; c = dvb-c et s = dvb-s (debut de normalisation?)
int devicemanager_api;
int devicemanager_vvideo;
int devicemanager_vvbi;
int devicemanager_tadapter=0;

#ifdef HAVE_XPM
  Pixmap icon_pixmap;
  Pixmap icon_shapemask;
#endif

/*------------------------------------------------------------------------*/

struct CALL_ACTION call_options_7 = { 0, "Options", {NULL} };

/*------------------------------------------------------------------------*/

enum {
  API,
  V4L_VIDEO,
  V4L_VBI,
  DVB_T_ADAPTER
};

// le système de switch entre les options v4l et tnt

static struct STRTAB* api_list;

// les listes qui seront dynamiques: ici statiques pour piger à quoi elles correspondent
static struct STRTAB* v4l_video_list;

static struct STRTAB* v4l_vbi_list;

static struct STRTAB* dvb_t_adapter_list;

/*-------------------------------------------------------------------------*/

void devman_switch_to_device(struct devman_device* mydevice){
	if(mydevice == NULL){
		fprintf(stderr, "Error, no device selected !\n");
		return;
	}

	if(debug) fprintf(stderr, "switching to %s\n", mydevice->name);

        
	if(strcmp("video", mydevice->precise_type) == 0){
		device.video = mydevice->device_location;
		devman_config_params->device_name = mydevice->name;
	}
	else if(strcmp("vbi", mydevice->precise_type) == 0)
		device.vbi = mydevice->device_location;

	if(debug) fprintf(stderr, "%s selected\n", mydevice->device_location);
}

/*-------------------------------------------------------------------------*/

void
switch_v4L_dvbt_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
#ifdef HAVE_DVB
	/* it only changes for dvb so we put it under ifdef */
	delay_between_failedframes=V4L_SLEEP_AFTER_FAILEDFRAME;
#endif
	grabbers[grabber]->grab_close();
	grabber = devicemanager_api;

	
	if((strcmp(api_list[devicemanager_api].str, "video4linux") == 0) || (strcmp(api_list[devicemanager_api].str, "v4l2") == 0)){
		if(debug) fprintf(stderr, "switching to analogic\n");
		devman_switch_to_device(devman_device_mathing_type_from_number("video", devicemanager_vvideo));
		devman_switch_to_device(devman_device_mathing_type_from_number("vbi", devicemanager_vvbi));
	}
#ifdef HAVE_DVB
	else if (strcmp(api_list[devicemanager_api].str,"dvb") == 0){
	  	delay_between_failedframes=DVB_SLEEP_AFTER_FAILEDFRAME;
		if(debug) fprintf(stderr, "switching to numeric\n");
	}
#endif
	grabber_init();
}

/*-------------------------------------------------------------------------*/

static void
devicemanager_menu_cb (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  long cd = (long) clientdata;
  int j;
  char label[sizeof(wchar_t)*64];

  switch (cd)
    {
    case API:
      if (-1 != (j = popup_menu (widget, translate("Choose your API"), api_list)))
        {
          devicemanager_api = j;
          sprintf (label, "%-*s: %s",
		   ALI(LAB_WIDTH, translate("Choose your API")),
                   int_to_str (devicemanager_api, api_list));
          XtVaSetValues (c_devicemanager_api, XtNlabel, label, NULL);
	if((strcmp(api_list[j].str, "video4linux") == 0) || (strcmp(api_list[j].str, "v4l2") == 0))
          {
	     if(debug) fprintf(stderr, "v4l 1/2 grabber selected\n");
             XtVaSetValues(c_devicemanager_vvideo, XtNsensitive,True,NULL);
             XtVaSetValues(c_devicemanager_vvbi, XtNsensitive,True,NULL);
             XtVaSetValues(c_devicemanager_tadapter, XtNsensitive,False,NULL);
          }
        else if (strcmp(api_list[j].str,"dvb") == 0)
          {
	     if(debug) fprintf(stderr, "dvb grabber selected\n");
             XtVaSetValues(c_devicemanager_vvideo, XtNsensitive,False,NULL);
             XtVaSetValues(c_devicemanager_vvbi, XtNsensitive,False,NULL);
             XtVaSetValues(c_devicemanager_tadapter, XtNsensitive,True,NULL);
          }
        }
      break;

    case V4L_VIDEO:
      if (-1 != (j = popup_menu (widget, translate("Video device"), v4l_video_list)))
        {
          devicemanager_vvideo = j;
          sprintf (label, "%-*s: %s",
		   ALI(LAB_WIDTH, translate("Video device")),
                   int_to_str (devicemanager_vvideo, v4l_video_list));
          XtVaSetValues (c_devicemanager_vvideo, XtNlabel, label, NULL);
        }
      break;

    case V4L_VBI:
      if (-1 != (j = popup_menu (widget, translate("Vbi device"), v4l_vbi_list)))
        {
          devicemanager_vvbi = j;
          sprintf (label, "%-*s: %s",
		   ALI(LAB_WIDTH, translate("Vbi device")),
                   int_to_str (devicemanager_vvbi, v4l_vbi_list));
          XtVaSetValues (c_devicemanager_vvbi, XtNlabel, label, NULL);
        }
      break;

    case DVB_T_ADAPTER:
      if (-1 != (j = popup_menu (widget, translate("DVB-T device"), dvb_t_adapter_list)))
        {
          devicemanager_tadapter = j;
          sprintf (label, "%-*s: %s",
		   ALI(LAB_WIDTH, translate("DVB-T device")),
                   int_to_str (devicemanager_tadapter, dvb_t_adapter_list));
          XtVaSetValues (c_devicemanager_tadapter, XtNlabel, label, NULL);
        }
      break;

    default:
      break;
    }
}

/*--------------------------------------------------------------------------*/

static void
close_devicemanagerwin (Widget widget, XtPointer clientdata, XtPointer call_data)
{
  devman_write_config_file();
  XtCallActionProc (devicemanager_shell, "DeviceManager", NULL, NULL, 0);
}

void
DeviceManagerAction (Widget widget, XEvent * event, String * params, Cardinal * num_params)
{
  static int mapped = 0, first = 1;
  
  if (event && event->type == ClientMessage)
    {
      if (event->xclient.data.l[0] == wm_protocols[1])
        {
          if (debug)
            fprintf (stderr, "Config: wm_save_yourself\n");
          XSetCommand (XtDisplay (devicemanager_shell), XtWindow (devicemanager_shell), NULL, 0);
          return;
        }
    }

  if (devicemanager_shell == NULL)
    {
      fprintf(stderr, "DeviceManagerAction: ooops: devicemanager_shell is NULL\n");
      return;
    }

  if (mapped)
    {
      XtPopdown (devicemanager_shell);
      mapped = 0;
    }
  else
    {
      XtPopup (devicemanager_shell, XtGrabNone);
      mapped = 1;
      if (first)
        {
          XSetWMProtocols (XtDisplay (devicemanager_shell), XtWindow (devicemanager_shell), wm_protocols, 2);
          first = 0;
        }
    }
}

/*--------------------------------------------------------------------------*/

void devman_initialize_menus(){
	int number_of_devices,i;
	struct devman_device_list* temp_devman_devices;

	/* V4L */

	/* First run to get the number of devices */
	temp_devman_devices = devman_device_matching_type("video", devman_devices);
	for(number_of_devices = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type("video", temp_devman_devices->next)) number_of_devices++;
	
	v4l_video_list = (struct STRTAB*)malloc((number_of_devices+1) * sizeof(struct STRTAB));

	temp_devman_devices = devman_device_matching_type("video", devman_devices);
	for(i = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type("video", temp_devman_devices->next)){
		v4l_video_list[i].nr = i;
		v4l_video_list[i].str = temp_devman_devices->current->name;
		i++;
	}
	v4l_video_list[i].nr = -1;
	v4l_video_list[i].str = NULL;


	/* VBI */
	temp_devman_devices = devman_device_matching_type("vbi", devman_devices);
	for(number_of_devices = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type("vbi", temp_devman_devices->next)) number_of_devices++;
	
	v4l_vbi_list = (struct STRTAB*)malloc((number_of_devices+2) * sizeof(struct STRTAB));

	temp_devman_devices = devman_device_matching_type("vbi", devman_devices);
	for(i = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type("vbi", temp_devman_devices->next)){
		v4l_vbi_list[i].nr = i;
		v4l_vbi_list[i].str = temp_devman_devices->current->name;
		i++;
	}
	v4l_vbi_list[i].nr = i;
	v4l_vbi_list[i].str = "None";
	i++;
	v4l_vbi_list[i].nr = -1;
	v4l_vbi_list[i].str = NULL;


	/* DVB */

	const char dvb_type[] = "frontend";

	/* First run to get the number of devices */
	temp_devman_devices = devman_device_matching_type(dvb_type, devman_devices);
	for(number_of_devices = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type(dvb_type, temp_devman_devices->next)) number_of_devices++;
	
	dvb_t_adapter_list = (struct STRTAB*)malloc((number_of_devices+1) * sizeof(struct STRTAB));

	temp_devman_devices = devman_device_matching_type(dvb_type, devman_devices);
	for(i = 0;temp_devman_devices != NULL ; temp_devman_devices = devman_device_matching_type(dvb_type, temp_devman_devices->next)){
		dvb_t_adapter_list[i].nr = i;
		dvb_t_adapter_list[i].str = temp_devman_devices->current->name;
		i++;
	}
	dvb_t_adapter_list[i].nr = -1;
	dvb_t_adapter_list[i].str = NULL;

	/* Interface list based on compiled grabbers */

	for(number_of_devices=0; grabbers != NULL && (grabbers[number_of_devices] != NULL);number_of_devices++);

	api_list = (struct STRTAB*) malloc( (number_of_devices+1) *sizeof(struct STRTAB));
  	for(i=0;i<number_of_devices;i++){
		api_list[i].nr = i;
		api_list[i].str = grabbers[i]->name;
		if(debug) fprintf(stderr, "adding grabber : %s\n", grabbers[i]->name);
	}
	api_list[i].nr = -1;
	api_list[i].str = NULL;

}

/*--------------------------------------------------------------------------*/

void
create_devicemanagerwin()
{
  char label[sizeof(wchar_t)*64];

  devman_initialize_menus();
  translate_struct(api_list);
  translate_struct(v4l_video_list);
  translate_struct(v4l_vbi_list);
  translate_struct(dvb_t_adapter_list);

  devicemanager_shell = XtVaAppCreateShell ("DeviceManager", "XdTV",
                                   topLevelShellWidgetClass,
                                   dpy, XtNclientLeader, app_shell, XtNvisual,vinfo->visual,
				   XtNcolormap,colormap, XtNdepth,vinfo->depth,
				   NULL);

  devicemanager_paned =
    XtVaCreateManagedWidget ("paned", panedWidgetClass, devicemanager_shell, NULL);

  /* Device Manager Label */
  c_devicemanager_label = XtVaCreateManagedWidget ("ldevicemanagerlabel", labelWidgetClass, devicemanager_paned,
                           PANED_FIX, NULL);

   /* Device Manager API Menu */
  c_devicemanager_api = XtVaCreateManagedWidget ("ldevicemanagerapi", commandWidgetClass, devicemanager_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_devicemanager_api, XtNcallback, devicemanager_menu_cb, (XtPointer) API);
  sprintf (label, "%-*s: %s", ALI(LAB_WIDTH, translate("Choose your API")), api_list[grabber].str);
  XtVaSetValues (c_devicemanager_api, XtNlabel, label, NULL);

////////////////////////////// V4L //////////////////////////////////////////////////////////////////////////
  /* Device Manager V4L Label */
  c_devicemanager_v4l = XtVaCreateManagedWidget ("ldevicemanagerv4l", labelWidgetClass, devicemanager_paned,
                           PANED_FIX, NULL);

   /* Device Manager V4L Video device */
  c_devicemanager_vvideo = XtVaCreateManagedWidget ("ldevicemanagervvideo", commandWidgetClass, devicemanager_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_devicemanager_vvideo, XtNcallback, devicemanager_menu_cb, (XtPointer) V4L_VIDEO);
  sprintf (label, "%-*s: %s", ALI(LAB_WIDTH, translate("Video device")), "------");
  XtVaSetValues (c_devicemanager_vvideo, XtNlabel, label, NULL);

 /* Device Manager V4L Vbi device */
  c_devicemanager_vvbi = XtVaCreateManagedWidget ("ldevicemanagervvbi", commandWidgetClass, devicemanager_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_devicemanager_vvbi, XtNcallback, devicemanager_menu_cb, (XtPointer) V4L_VBI);
  sprintf (label, "%-*s: %s", ALI(LAB_WIDTH, translate("Vbi device")), "------");
  XtVaSetValues (c_devicemanager_vvbi, XtNlabel, label, NULL);


///////////////////////////// DVBT //////////////////////////////////////////////////////////////////////////

  /* Device Manager TNT Label */
  c_devicemanager_dvbt = XtVaCreateManagedWidget ("ldevicemanagerdvbt", labelWidgetClass, devicemanager_paned,
                           PANED_FIX, NULL);

 /* Device Manager DVBT Adapter device */
  c_devicemanager_tadapter = XtVaCreateManagedWidget ("ldevicemanagertadapter", commandWidgetClass, devicemanager_paned,
                               PANED_FIX, NULL);
  XtAddCallback (c_devicemanager_tadapter, XtNcallback, devicemanager_menu_cb, (XtPointer) DVB_T_ADAPTER);
  sprintf (label, "%-*s: %s", ALI(LAB_WIDTH, translate("DVB-T device")), "------");
  XtVaSetValues (c_devicemanager_tadapter, XtNlabel, label, NULL);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/* Device Manager switch Action */
  c_devicemanager_switch = XtVaCreateManagedWidget ("ldevicemanagerswitch", commandWidgetClass, devicemanager_paned, PANED_FIX, NULL);
    XtAddCallback (c_devicemanager_switch, XtNcallback, switch_v4L_dvbt_cb, (XtPointer) NULL);

/* Open Main Options Menu */
  c_devicemanager_openmain = XtVaCreateManagedWidget ("ldevicemanageropenmain", commandWidgetClass, devicemanager_paned, PANED_FIX, NULL);
    XtAddCallback (c_devicemanager_openmain, XtNcallback, action_cb, (XtPointer) & call_options_7);

/* Button to close Device Manager Parameters GUI */
  c_devicemanager_save = XtVaCreateManagedWidget ("ldevicemanagersave", commandWidgetClass, devicemanager_paned, PANED_FIX, NULL);
  XtAddCallback (c_devicemanager_save, XtNcallback, save_cb, (XtPointer) NULL);
  XtAddCallback (c_devicemanager_save, XtNcallback, close_devicemanagerwin, (XtPointer) NULL);

#ifdef HAVE_XPM
  XpmCreatePixmapFromData(XtDisplay(devicemanager_shell),
                          RootWindowOfScreen(XtScreen(devicemanager_shell)),
                          icon_xpm,
                          &icon_pixmap,
                          &icon_shapemask, NULL);
  XtVaSetValues(devicemanager_shell, XtNiconPixmap, icon_pixmap, NULL);
#endif

  // POUR PLUS TARD QUAND CE SERA STABILISE
  //create_devicemanager_pixmaps(XtDisplay(devicemanager_paned),&RootWindowOfScreen(XtScreen(devicemanager_paned)));
}
